/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cameo/cameo.h>
#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <thread>

using namespace std;
using namespace cameo;

int main(int argc, char *argv[]) {

	application::This::init(argc, argv);

	// New block to ensure cameo objects are terminated before the application.
	{
		string applicationName;

		int numberOfSubscribers = 1;

		if (argc > 2) {
			applicationName = argv[1];
			cout << "subscriber application is " + applicationName << endl;

			if (argc > 3) {
				numberOfSubscribers = stoi(argv[2]);
			}

		} else {
			cerr << "arguments: [subscriber application name] [number of subscribers]" << endl;
			return -1;
		}

		// get the client services
		Server& server = application::This::getServer();

		if (application::This::isAvailable() && server.isAvailable()) {
			cout << "connected server " << server << endl;
		}

		// loop the number of subscribers.
		for (int i = 0; i < numberOfSubscribers; ++i) {

			// pass the name of the application in argument
			vector<string> applicationArgs;
			applicationArgs.push_back(application::This::getName());

			// start the application.
			unique_ptr<application::Instance> subscriberApplication = server.start(applicationName, applicationArgs);

			if (subscriberApplication->exists()) {
				cout << "started application " << *subscriberApplication << endl;
			}
			else {
				cout << "cannot start subscriber application" << endl;
			}
		}

		// the publisher is created after the applications that will wait for it
		unique_ptr<application::Publisher> publisher = application::Publisher::create("publisher");

		int j = 0;
		while (j < 20) {

			string data = "{";
			data += to_string(j) + ", " + to_string(j * j) + "}";
			publisher->send(data);

			cout << "sent " << data << endl;

			this_thread::sleep_for(chrono::milliseconds(100));

			j++;
		}

		// the publisher must terminate so that the subscriber applications receive end of stream
		publisher->sendEnd();

		// wait for the end of the applications
		application::InstanceArray instances = server.connectAll(applicationName);

		for (int i = 0; i < instances.size(); i++) {
			instances[i]->waitFor();
		}

		cout << "finished the application" << endl;
	}

	return 0;
}
