/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.test;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.RemoteException;
import fr.ill.ics.cameo.Server;


public class TestAsyncResponderAndRequesterApplication {

	public static void main(String[] args) {
		
		Application.This.init(args);
		
		String applicationName = null;
		int numberOfTimes = 1;
		
		if (args.length > 1) {
			applicationName = args[0];
			System.out.println("requester application is " + applicationName);
			
			if (args.length > 2) {
				numberOfTimes = Integer.parseInt(args[1]);
			}
			
		} else {
			System.err.println("arguments: [application name]");
			System.exit(-1);
		}
		
		// get the client services
		Server server = Application.This.getServer();
		
		if (Application.This.isAvailable() && server.isAvailable()) {
			System.out.println("connected application");
			System.out.println("connected server " + server);
		} else {
			System.exit(-1);
		}
		
		try {
			// Create the responder.
			Application.Responder responder = Application.Responder.create("responder");
			
			// Set the state.
			Application.This.setRunning();
			
			int N = 5;
			
			// loop the number of times.
			for (int i = 0; i < numberOfTimes; ++i) {

				Application.Instance[] requesterApps = new Application.Instance[N];
				
				// Start the requester applications.
				for (int j = 0; j < N; ++j) {
					
					// Start the application.
					requesterApps[j] = server.start(applicationName);
					System.out.println("started application " + requesterApps[j]);
				}
				
				Thread[] threads = new Thread[N];
				
				// Process the requests.
				for (int j = 0; j < N; ++j) {
					
					// Receive the simple request.
					Application.Request request = responder.receive();
					
					threads[j] = new Thread(new Runnable() {
					    @Override
					    public void run() {
							// Reply.
				    		request.reply("done");
							
							// Terminate the request.
							request.terminate();
					    }
					});
					
					threads[j].start();
				}
				
				// Wait for the requester applications.
				for (int j = 0; j < N; ++j) {
					requesterApps[j].waitFor();
					System.out.println("finished application " + requesterApps[j]);
					
					try {
						threads[j].join();
						
					} catch (InterruptedException e) {
					}
				}
			}
			
		} catch (RemoteException e) {
			System.out.println("requester error:" + e);
			
		} finally {
			// Do not forget to terminate This.
			Application.This.terminate();
		}
		
		System.out.println("finished the application");
	}

}