/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.test;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.RemoteException;
import fr.ill.ics.cameo.Server;
import fr.ill.ics.cameo.messages.Message;


public class TestRequesterAndResponderApplication {

	public static void main(String[] args) {
		
		Application.This.init(args);
		
		String applicationName = null;
		int numberOfTimes = 1;
		
		if (args.length > 1) {
			applicationName = args[0];
			System.out.println("responder application is " + applicationName);
			
			if (args.length > 2) {
				numberOfTimes = Integer.parseInt(args[1]);
			}
			
		} else {
			System.err.println("arguments: [application name]");
			System.exit(-1);
		}
		
		// get the client services
		Server server = Application.This.getServer();
		
		if (Application.This.isAvailable() && server.isAvailable()) {
			System.out.println("connected application");
			System.out.println("connected server " + server);
		} else {
			System.exit(-1);
		}
		
		try {
			
			// loop the number of times.
			for (int i = 0; i < numberOfTimes; ++i) {
			
				// Start the application.
				Application.Instance responderApplication = server.start(applicationName);
				System.out.println("started application " + responderApplication + " with state " + Application.State.toString(responderApplication.getActualState()));

				Application.Requester requester = Application.Requester.create(responderApplication, "responder");
				System.out.println("created requester " + requester);
			
				// Check the state of the responder app.
				System.out.println("application " + responderApplication + " has state " + Application.State.toString(responderApplication.getActualState()));
				
				
				// Send a simple message.
				requester.send("request");
				System.out.println("response is " + requester.receiveString());
			
				
				// Send a two-parts message.
				requester.sendTwoParts(Message.serialize("first"), Message.serialize("second"));
				System.out.println("response is " + requester.receiveString());
				
				
				// Send a simple message.
				requester.send("request");
				
				// Wait 1s.
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
				
				System.out.println("response is " + requester.receiveString());
				
				
				// Send a simple message.
				requester.send("request");
				System.out.println("response is " + requester.receiveString());
				
				
				int state = responderApplication.waitFor();
				System.out.println("responder application terminated with state " + Application.State.toString(state));
				
				// Terminate the requester.
				requester.terminate();
			}
			
		} catch (RemoteException e) {
			System.out.println("requester error:" + e);
			
		} finally {
			// Do not forget to terminate This.
			Application.This.terminate();
		}
		
		System.out.println("finished the application");
	}

}